#region USING
using System;
using System.Data;
using System.Xml;
using System.Xml.Xsl;
using System.Xml.XPath;
#endregion
namespace RSSConnect
{
	/// <summary>
	/// Abstract class for database access
	/// </summary>
	public abstract class DBHelperBase
	{
		#region PUBLIC_MEMBERS
		public string ConnectionString
		{
			get
			{
				return connectionString;
			}

			set
			{
				if(value != connectionString)
					connectionString = value;
			}
		}

		public string LastError
		{
			get
			{
				return lastError;
			}

			set
			{
				if(value != lastError)
					lastError = value;
			}
		}

		//	Returns two numbers: entries processed and records inserted
		//	Input is the local XML file name from where the data is to be imported
		public int [] ExportFeedToDB(string feedSource)
		{
			int[] result = {0,0};
			try
			{
				//	Earlier used XPathDocument, but if the XML contains external references, then
				//	on .NET Framework 1.1 on Win2K Advanced Server, it again raised SocketException
				XmlDocument rssXMLDoc = new XmlDocument();
				//	Important to set XmlResolver to null, so that no external references are resolved
				rssXMLDoc.XmlResolver = null;

				rssXMLDoc.Load(feedSource);
				XPathNavigator feedNav = rssXMLDoc.CreateNavigator();
				XmlNamespaceManager xmlnsManager = new XmlNamespaceManager(feedNav.NameTable);
				xmlnsManager.AddNamespace("content", "http://purl.org/rss/1.0/modules/content/");
				xmlnsManager.AddNamespace("xhtml", "http://www.w3.org/1999/xhtml");
				xmlnsManager.AddNamespace("slash", "http://purl.org/rss/1.0/modules/slash/");
				xmlnsManager.AddNamespace("dc", "http://purl.org/dc/elements/1.1/");

				try
				{
					result[0] = Convert.ToInt32(feedNav.Evaluate("count(//item)"));
					DebugLog.Instance.DEBUG1(result[0].ToString() + " items found!");
				}
				catch(Exception exp) {DebugLog.Instance.REPORT(exp);}

				XPathNodeIterator feedIter = feedNav.Select("//item");

				//	Number of records inserted
				result[1] = InsertRecords(feedIter, xmlnsManager);
			}
			catch(Exception exp) {DebugLog.Instance.REPORT(exp);}
			return result;
		}

		public abstract bool TestConnection();
		public abstract DataSet Search(string searchCriteria, searchTypeEnum searchType);
		public abstract int IndexDB();
		#endregion
		#region PROTECTED_MEMBERS
		protected string connectionString;
		protected string lastError="";
		protected abstract int InsertRecords(XPathNodeIterator feedIter, XmlNamespaceManager nsManager);
		#endregion
	}
}
