#region USING
using System;
using System.IO;
using System.Configuration;
using System.Diagnostics;
using System.Management;
#endregion
namespace RSSConnect
{
	#region COMMENTS
	/// <summary>
	/// Class used to generate log information useful for debugging
	///	debugLevel == 0 No debugging
	///	debugLevel == 1	basic debugging (logs DEBUG1 function calls)
	///	debugLevel == 3	intermediate debugging info (logs DEBUG2 function calls)
	///	debugLevel == 3	basic debugging (logs DEBUG3 function calls)

	///	Use REPORT function for error reporting - always logged
	///	Set the debugLevel in the app configuration file (key named DebugLogLevel)
	/// </summary>
	#endregion
	sealed class DebugLog
	{
		public static readonly DebugLog Instance = new DebugLog();

		#region PRIVATE_MEMBERS
		private static int debugLevel;
		private static StreamWriter streamWriter;
		#endregion
		#region CTOR
		private DebugLog()
		{
		}
		#endregion

		#region PUBLIC_METHODS
		public void BeginLog(string InstanceFileName)
		{
			debugLevel = 0;
			try
			{
				string debugLogConfigVal = ConfigurationSettings.AppSettings["DebugLogLevel"]; 
				try
				{
					if(debugLogConfigVal != null)
						debugLevel = Convert.ToInt32(debugLogConfigVal);
				}
				catch(Exception) {debugLevel=0;}

				//	Make a backup copy of last/existing log file (if any)
				try
				{
					if(File.Exists(InstanceFileName))
						File.Copy(InstanceFileName, InstanceFileName + ".PREV.log", true);
				}
				catch(Exception) {}

				streamWriter = new StreamWriter(InstanceFileName, false);
				WriteHeader();
			}
			catch(Exception)
			{
			}
		}

		public void EndLog()
		{
			try
			{
				if(streamWriter != null)
				{
					streamWriter.WriteLine("-----------------------------------------------------");
					streamWriter.WriteLine("Exiting: " + System.DateTime.Now.ToString());
					streamWriter.Close();
				}
			}
			catch(Exception)
			{
			}
		}
			
		public void DEBUG1(string strLogText)
		{
			try
			{
				if(streamWriter != null && debugLevel >= 1) 
				{
					streamWriter.WriteLine("{0}\t{1}", DateTime.Now.ToLongTimeString(), 
						strLogText);
					streamWriter.Flush();
				}
			}
			catch(Exception)
			{
			}
		}
		
		public void DEBUG2(string strLogText)
		{
			try
			{
				if(streamWriter != null && debugLevel > 1)
				{
					streamWriter.WriteLine("{0}\t{1}", DateTime.Now.ToLongTimeString(), 
						strLogText);
					streamWriter.Flush();
				}
			}
			catch(Exception)
			{
			}
		}

		public void DEBUG3(string strLogText)
		{
			try
			{
				if(streamWriter != null && debugLevel > 2)
				{
					streamWriter.WriteLine("{0}\t{1}", DateTime.Now.ToLongTimeString(), 
						strLogText);
					streamWriter.Flush();
				}
			}
			catch(Exception)
			{
			}
		}

		public void REPORT(string strLogText)
		{
			try
			{
				if(streamWriter != null)
				{
					streamWriter.WriteLine("{0}\t{1}", DateTime.Now.ToLongTimeString(), 
						strLogText);
					streamWriter.Flush();
				}
			}
			catch(Exception){}
		}

		public void REPORT(Exception exp)
		{
			REPORT("*** EXCEPTION: " + exp.ToString());
		}

		#endregion

		#region PRIVATE_METHODS
		private void WriteHeader()
		{
			try
			{
				if(streamWriter != null)
				{
					streamWriter.WriteLine("RSSConnect by PerfectXML.com");
					streamWriter.WriteLine("----------------------------");
					streamWriter.WriteLine("DebugLogLevel: " + debugLevel.ToString());
					streamWriter.WriteLine("App Started: " + System.DateTime.Now.ToString());
					streamWriter.WriteLine("-----------------------------------------------------");

					//	For detailed debugging, log some of the system information
					if(debugLevel > 2)
					{
						try
						{
							ManagementClass wmiClass = new ManagementClass("Win32_OperatingSystem");
							ManagementObjectCollection wmiObjectColl = wmiClass.GetInstances();
							foreach(ManagementObject wmiObject in wmiObjectColl)
							{
								streamWriter.WriteLine(wmiObject.GetPropertyValue("Caption"));
								streamWriter.Write("(" + wmiObject.GetPropertyValue("CSDVersion") + ", ");
								streamWriter.Write(wmiObject.GetPropertyValue("Version") + ", ");
								streamWriter.Write(wmiObject.GetPropertyValue("CodeSet") + ", ");
								streamWriter.Write(wmiObject.GetPropertyValue("CSName") + ", ");
								streamWriter.Write(wmiObject.GetPropertyValue("OSLanguage")+ ", ");
								streamWriter.Write(wmiObject.GetPropertyValue("BuildNumber")+ ", ");
								streamWriter.Write(wmiObject.GetPropertyValue("CountryCode")+ ", ");
								streamWriter.WriteLine(wmiObject.GetPropertyValue("Locale")+ ")");
								streamWriter.WriteLine("-------------------------------------------------");
							}
						}
						catch(Exception){}
					}
					streamWriter.Flush();
				}
			}
			catch(Exception){}
		}
		#endregion
	
	}	//	sealed class DebugLog
}	//	namespace RSSConnect
