#region USING
using System;
using System.Xml;
using System.Xml.Xsl;
using System.Xml.XPath;

using System.Collections;
using System.IO;

using System.Data;
using System.Data.OleDb;
#endregion
namespace RSSConnect
{
	/// <summary>
	/// OLEDB Data Access 
	/// </summary>
	public class OLEDBHelper: DBHelperBase
	{
		#region CTOR
		public OLEDBHelper(string connectionString)
		{
			this.connectionString = connectionString;
		}

		#endregion
		#region PUBLIC_METHODS
		public override bool TestConnection()
		{
			//	Returns true or false indicating if connection to DB was successful or not
			bool result = false;
			OleDbConnection conn = new OleDbConnection();
			try
			{
				if(connectionString != null && connectionString.Length > 0)
				{
					conn.ConnectionString = connectionString;
					conn.Open();
					result = true;
				}
			}
			catch(Exception exp) 
			{
				lastError = exp.Message;
				DebugLog.Instance.REPORT(exp);
			}
			finally
			{
				try {if(conn.State == ConnectionState.Open) conn.Close();}
				catch(Exception) {}
			}

			return result;
		}
		
		public override DataSet Search(string searchCriteria, searchTypeEnum searchType)
		{
			//	Returns DataSet containing search results
			DataSet result = new DataSet("RSSConnectSearchResults");

			OleDbConnection conn = new OleDbConnection(connectionString);
			DebugLog.Instance.DEBUG3("Search started...");
			try
			{
				conn.Open();

				Utility utility = new Utility();
				string mainSqlStatement = "SELECT [Title], [Date], [Link], [Description] FROM [tblFeeds] WHERE [RowID] IN (";
				string IndexSqlStatement = "SELECT [FeedsRowIDs] FROM [tblIndex] WHERE [Word] = ";
				string stopWords="";

				string[] words = searchCriteria.Split(AppOptions.Instance.wordSep);

				bool allStopWords = true;
				string resultRowIDs = "";

				foreach(string word in words)
				{
					try
					{
						if(word != null && word.Length > 1)
						{
							if(AppOptions.Instance.stopWords[word.ToLower()] != null)
								stopWords += word + " ";
							else
							{
								allStopWords = false;
								 
								OleDbCommand comm = new OleDbCommand(IndexSqlStatement + "'" + word + "'", conn);
								string curResult = Convert.ToString(comm.ExecuteScalar());

								if(resultRowIDs.Length <= 0)
								{//	first word
									resultRowIDs = curResult;
								}
								else
								{
									if(searchType == searchTypeEnum.anyWords)
									{
										resultRowIDs += curResult;
									}
									else
									{
										//	all words search type
										string mergedRowIDs=""; 
										string [] resultRowIDsArray = resultRowIDs.Split(',');
										string [] curResultRowIDsArray = curResult.Split(',');
										foreach(string rowID in resultRowIDsArray)
										{
											if(rowID != null && rowID.Length > 0)
											{
												foreach(string curRowID in curResultRowIDsArray)
												{
													if(curRowID != null && curRowID.Length > 0)
													{
														try
														{
															if(Convert.ToInt32(curRowID) == Convert.ToInt32(rowID))
															{
																mergedRowIDs += curRowID + ",";
																break;
															}
														}
														catch(Exception exp) {DebugLog.Instance.REPORT(exp);}
													}
												}
											}
										}
										resultRowIDs = mergedRowIDs;
									}
								}
							}
						}
					}
					catch(Exception exp) {DebugLog.Instance.REPORT(exp);}
				}

				if(!allStopWords && resultRowIDs != null)
				{
					int strLength = resultRowIDs.Length;
					if(strLength > 1)
					{
						resultRowIDs = resultRowIDs.Substring(0, strLength-1);
						mainSqlStatement += resultRowIDs + ")";
						OleDbDataAdapter adapter = new OleDbDataAdapter(mainSqlStatement, conn);
						adapter.Fill(result);
					}
				}
				
				DataTable summaryTable = result.Tables.Add("Summary");
				summaryTable.Columns.Add("SearchText");
				summaryTable.Columns.Add("StopWords");
				summaryTable.Columns.Add("SearchDate");

				DataRow summaryRow = summaryTable.NewRow();
				summaryRow[0] = searchCriteria;
				summaryRow[1] = stopWords;
				summaryRow[2] = System.DateTime.Now.ToString();
				summaryTable.Rows.Add(summaryRow);

				DebugLog.Instance.DEBUG3("Search over.");
			}
			catch(Exception exp) 
			{
				lastError = exp.Message;
				DebugLog.Instance.REPORT(exp);
			}
			finally
			{
				try {if(conn.State == ConnectionState.Open) conn.Close();}
				catch(Exception) {}
			}
			return result;
		}
		
		public override int IndexDB()
		{
			//	Updates the database index ([tblIndex])
			DebugLog.Instance.DEBUG3("Database Indexing Started.");
			int result = 0;
			OleDbConnection conn = new OleDbConnection(connectionString);
			OleDbConnection updateCmdConn = new OleDbConnection(connectionString);

			string wordRecordPresentQuery = "SELECT COUNT([Word]) FROM [tblIndex] WHERE [Word] = '";
			try
			{
				conn.Open();
				updateCmdConn.Open();

				OleDbCommand updateComm = new OleDbCommand();
				updateComm.Connection = updateCmdConn;

				//	1. Get a list of records where Indexed = 0 (that is records that are not yet indexed)
				OleDbCommand comm = new OleDbCommand
					("SELECT [RowID], [Title], [Description] FROM [tblFeeds] WHERE Indexed = 0", conn);

				OleDbDataReader dataReader = comm.ExecuteReader();
				Utility utility = new Utility();
				while(dataReader.Read())
				{
					try
					{
						int rowID = dataReader.GetInt32(0);

						string title = dataReader.GetString(1);
						if(title == null)
							title = "";

						string desc = dataReader.GetString(2);
						if(desc == null)
							desc = "";

						Hashtable indexWords = utility.GetIndexWords("" + title + " " + desc);

						foreach(string aWord in indexWords.Keys)
						{
							try
							{
								updateComm.CommandText = wordRecordPresentQuery + aWord.Trim() + "'";
								if(Convert.ToInt32(updateComm.ExecuteScalar()) <= 0)
								{
									updateComm.CommandText = 
										string.Format("INSERT INTO [tblIndex] ([Word], [FeedsRowIDs]) VALUES ('{0}', '{1},')",
										aWord.Trim(), rowID.ToString());
								}
								else
								{
									//	AS + operator does not work with blob text fields, get [FeedsRowIDs] first,
									//	update the string, and then set the new value
									updateComm.CommandText = "SELECT [FeedsRowIDs] FROM [tblIndex] WHERE [Word] ='" + aWord.Trim() + "'";
									string feedRowIDs = Convert.ToString(updateComm.ExecuteScalar());
									feedRowIDs += rowID.ToString() + ",";
									updateComm.CommandText = 
										string.Format("UPDATE [tblIndex] SET [FeedsRowIDs] = '{0}' WHERE [Word] = '{1}'",
										feedRowIDs, aWord.Trim());
								}
								updateComm.ExecuteNonQuery();
							}
							catch(Exception exp) {DebugLog.Instance.REPORT(exp);}
						}
	
						updateComm.CommandText = 
							string.Format("UPDATE [tblFeeds] SET [Indexed] = 1 WHERE [RowID] = {0}",
							rowID.ToString());
						updateComm.ExecuteNonQuery();
						result++;
					}
					catch(Exception exp) {DebugLog.Instance.REPORT(exp);}
				}

			}
			catch(Exception exp) 
			{
				lastError = exp.Message;
				DebugLog.Instance.REPORT(exp);
			}
			finally
			{
				try {if(conn.State == ConnectionState.Open) conn.Close();}
				catch(Exception) {}
				try {if(updateCmdConn.State == ConnectionState.Open) updateCmdConn.Close();}
				catch(Exception) {}
			}

			DebugLog.Instance.DEBUG3("Database Indexing Complete. " + result.ToString() + " records indexed.");
			return result;
		}

		#endregion
		#region PROTECTED_METHODS
		protected override int InsertRecords(XPathNodeIterator feedIter, XmlNamespaceManager nsManager)
		{
			int result = 0;
			OleDbConnection conn = new OleDbConnection(connectionString);
			try
			{
				conn.Open();
				while(feedIter.MoveNext())
				{
					try
					{
						XPathNavigator curRowNav = feedIter.Current; 
						string feedTitle="", feedDate="", feedLink="", feedDesc="";

						try
						{
							feedLink = Convert.ToString(curRowNav.Evaluate("string(link)")).Replace("'", "''");
							if(feedLink.Length <= 0)
								feedLink = Convert.ToString(curRowNav.Evaluate("string(guid)")).Replace("'", "''");
						}
						catch(Exception) {}

						OleDbCommand comm = new OleDbCommand();
						comm.Connection = conn;

						//	Make sure feedLink is not already in the database
						string selectQuery = "SELECT COUNT([RowID]) FROM [tblFeeds] WHERE [Link] = '" + feedLink + "'";
						comm.CommandText = selectQuery;

						if (Convert.ToInt32(comm.ExecuteScalar()) <= 0)
						{
							try
							{
								feedTitle = Convert.ToString(curRowNav.Evaluate("string(title)")).Replace("'", "''");
							}
							catch(Exception) {}

							try
							{
								XPathExpression xpathExpr;

								xpathExpr = curRowNav.Compile("string(pubDate)");
								feedDate = Convert.ToString(curRowNav.Evaluate(xpathExpr)).Replace("'", "''");
								if(feedDate.Length <= 0)
								{
									xpathExpr = curRowNav.Compile("string(dc:date)");
									xpathExpr.SetContext(nsManager);
									feedDate = Convert.ToString(curRowNav.Evaluate(xpathExpr)).Replace("'", "''");
								}
							}
							catch(Exception) {}

							try
							{
								XPathExpression xpathExpr;

								xpathExpr = curRowNav.Compile("string(xhtml:body)");
								xpathExpr.SetContext(nsManager);
								feedDesc = Convert.ToString(curRowNav.Evaluate(xpathExpr)).Replace("'", "''");
								if(feedDesc.Length <= 0)
								{
									xpathExpr = curRowNav.Compile("string(content:encoded)");
									xpathExpr.SetContext(nsManager);
									feedDesc = Convert.ToString(curRowNav.Evaluate(xpathExpr)).Replace("'", "''");
									if(feedDesc.Length <= 0)
									{
										xpathExpr = curRowNav.Compile("string(description)");
										feedDesc = Convert.ToString(curRowNav.Evaluate(xpathExpr)).Replace("'", "''");
									}
								}
							}
							catch(Exception) {}

							string insertStmt;
							insertStmt = "INSERT INTO [tblFeeds] ([Title], [Date], [Link], [Description], [Indexed]) VALUES ('" +
								feedTitle + "', '"+
								feedDate  + "', '"+
								feedLink  + "', '"+
								feedDesc  + "', 0)";

							comm.CommandText = insertStmt;
							//DebugLog.Instance.DEBUG3(insertStmt);
							result += comm.ExecuteNonQuery();
						}
						else
						{
							DebugLog.Instance.DEBUG3(feedLink + " is already in the database.");
						}
					}
					catch(Exception exp) {DebugLog.Instance.REPORT(exp);}
				}
			}
			catch(Exception exp) 
			{
				lastError = exp.Message;
				DebugLog.Instance.REPORT(exp);
			}
			finally
			{
				try {if(conn.State == ConnectionState.Open) conn.Close();}
				catch(Exception) {}
			}

			return result;
		}

		#endregion
	}
}
