#region USING
using System;
using System.Windows.Forms;
using System.Xml;
using System.Collections;
using System.IO;
using System.Text;
#endregion
namespace RSSConnect
{
	#region COMMENTS
	/// <summary>
	/// Options.Xml Wrapper
	/// </summary>
	#endregion
	sealed class AppOptions
	{
		public static readonly AppOptions Instance = new AppOptions();
		#region PUBLIC_MEMBERS
		//	Second character is the tab character
		public char[] wordSep = {' ','	','.', ':', ';',',','!','@','#','$','%','^','(',')','+','=','-','>','<','?','\'','"','[',']','{','}','|','\\','/','`','','','','&','*','_','~','','\n',''};
		public Hashtable stopWords;
		#endregion
		#region PRIVATE_MEMBERS
		private XmlDocument xmlDoc;
		private string optionsXMLFileName="OptionsVer1.1.xml";
		#endregion
		#region CTOR
		private AppOptions()
		{
			try
			{
				xmlDoc = new XmlDocument();
				xmlDoc.Load(optionsXMLFileName);

				LoadStopWords();
			}
			catch(Exception exp) {DebugLog.Instance.REPORT(exp);}
		}
		#endregion
		#region PUBLIC_METHODS
		public string GetOption(string keyName)
		{
			string optionValue="";
			try
			{
				if(xmlDoc != null)
				{
					XmlNode optionNode = xmlDoc.SelectSingleNode
						("/options/option[@key='" + keyName + "']/@value");
					if(optionNode != null)
						optionValue = optionNode.Value.ToString();
				}
			}
			catch(Exception exp) {DebugLog.Instance.REPORT(exp);}
			return optionValue;
		}

		public bool SetOption(string keyName, string optionValue)
		{
			bool result = false;
			try
			{
				if(xmlDoc != null)
				{
					XmlNode optionNode = xmlDoc.SelectSingleNode
						("/options/option[@key='" + keyName + "']/@value");
					if(optionNode != null)
						optionNode.Value = optionValue.ToString();
					xmlDoc.Save(optionsXMLFileName);
					result = true;
				}
			}
			catch(Exception exp) {DebugLog.Instance.REPORT(exp);}

			return result;
		}
		#endregion
		#region PRIVATE_MEMBERS
		private void LoadStopWords()
		{
			StreamReader stopWordsFileReader = null;
			try
			{
				stopWords = new Hashtable(400);	//	initial hashtable size
				stopWordsFileReader = new StreamReader(Application.StartupPath + @"\StopWords.txt");
				string aWord;
				while((aWord = stopWordsFileReader.ReadLine()) != null)
				{
					if(aWord.Length > 0)
					{
						try
						{
							stopWords.Add(aWord, aWord);
						}
						catch(Exception) {}
					}
				}
			}
			catch(Exception exp) {DebugLog.Instance.REPORT(exp);}
			finally 
			{
				if(stopWordsFileReader != null)
				{
					try { stopWordsFileReader.Close();}
					catch(Exception) {}
				}
			}
		}
		#endregion
	}	
}	
